#include <sta/devices/arduino/bus/i2c.hpp>

#ifdef STA_PLATFORM_ARDUINO

#include <sta/devices/arduino/hal.hpp>


namespace sta
{
    ArduinoI2C::ArduinoI2C(Mutex * mutex /* = nullptr */)
        : I2C{mutex}
    {
        Wire.begin();
    }

    void ArduinoI2C::transfer(uint8_t value)
    {
        Wire.beginTransmission(address_);
        Wire.write(value);
        Wire.endTransmission(true);
    }

    void ArduinoI2C::transfer16(uint16_t value)
    {
        Wire.beginTransmission(address_);
        Wire.write(value);
        Wire.endTransmission(true);
    }

    void ArduinoI2C::transfer(const uint8_t * buffer, size_t size)
    {
        Wire.beginTransmission(address_);
        Wire.write(buffer, size);
        Wire.endTransmission();
    }

    void ArduinoI2C::receive(uint8_t * buffer, size_t size)
    {
        Wire.requestFrom(address_, size);
        size_t index = 0;

        while (index < size)
        {
            while (Wire.available())
            {
                buffer[index++] = Wire.read();
            }
        }
    }

    void ArduinoI2C::fill(uint8_t value, size_t count)
    {
        // Initialize a buffer of size count and fill it with the value.
        uint8_t *buffer = new uint8_t[count];
        memset(buffer, value, count);

        Wire.beginTransmission(address_);
        Wire.write(buffer, count);
        Wire.endTransmission();

        delete[] buffer; 
    }

    ArduinoI2CDevice::ArduinoI2CDevice(I2C * intf, uint8_t address, bool master /* = true */, bool blocking /* = true */)
        : I2CDevice{intf, address, master, blocking}
    {

    }
} // namespace sta

#endif // STA_PLATFORM_ARDUINO